unit FormeF2dansF1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, math, Buttons;

type
  TForm1 = class(TForm)
    Image1: TImage;
    Panel1: TPanel;
    bTracer: TSpeedButton;
    bPremierTest: TSpeedButton;
    SpeedButton1: TSpeedButton;
    procedure bTracerClick(Sender: TObject);
    procedure bPremierTestClick(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    { Dclarations prives }
  public
    { Dclarations publiques }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

// Petits utilitaires : ========================================================

function RotationPoint(const Centre, Pt: tPoint; dPhiRad: Single): tPoint;
var Rxx, Ryy: Single; si, co: Extended;
begin
  Rxx := Pt.x - Centre.x; Ryy := Pt.y - Centre.y;
  // Application de la matrice de rotation par rapport au centre xc,yc
  SinCos(dPhiRad, si, co);
  Result.x := round(Centre.x + (Rxx * co - Ryy * si));
  Result.y := round(Centre.y + (Rxx * si + Ryy * co));
end;

procedure TranslaterPoint(var Point: TPoint; dx, dy: integer);
begin
  Point.X := Point.X + dx; Point.Y := Point.Y + dy;
end;

type TArrayOfTPoint = array of TPoint;

procedure TranslaterPolygone(var Poly: TArrayOfTPoint; dx, dy: integer);
var i: integer;
begin
  for i := 0 to High(Poly) do TranslaterPoint(Poly[i], dx, dy);
end;

procedure RotaPolygone(var Poly: TArrayOfTPoint; Centre: tPoint; IncliDeg: Extended);
var j, xc, yc: integer;
  Rxx, Ryy: Single; Si, Co: Extended;
begin
  xc := Centre.x; yc := Centre.y; SinCos(DegToRad(IncliDeg), Si, Co);
  for j := 0 to High(Poly) do begin
    Rxx := Poly[j].x - xc; Ryy := Poly[j].y - yc;
    // Application de la matrice de rotation par rapport au centre de coordonnes xc,yc
    Poly[j].x := round(xc + (Rxx * Co - Ryy * Si));
    Poly[j].y := round(yc + (Rxx * Si + Ryy * Co));
  end;
end;

function arcTan22(Y, X: Extended): Extended;
begin if X = 0 then begin
    if Y > 0 then Result := +Pi / 2 else
      if Y < 0 then Result := -Pi / 2 else begin Result := 0.0;
    end;
    EXIT;
  end;
  Result := arcTan2(Y, X);
end;

function PointDansPolygone(const Point: TPoint; Polygone: TArrayOfTPoint): boolean; //array of TPoint): boolean;
//       Renvoie True si le point x,y est  l'intrieur du Polygone de points
var Handle: HRGN;
begin
  Handle := CreatePolygonRgn(Polygone[0], length(Polygone), Winding);
  Result := PtInRegion(Handle, Point.X, Point.Y);
  DeleteObject(Handle);
end;

procedure TracerPoly(SurBmp: TBitMap; Poly: TArrayOfTPoint; Coul: TColor; EpTrait: byte);
var i: integer;
begin
  with SurBmp.Canvas do begin
    Brush.Style := bsClear; Pen.Width := EpTrait; Pen.Color := Coul; Pen.Mode := pmCopy;
    moveTo(Poly[0].X, Poly[0].Y);
    for i := 1 to high(Poly) do begin
      lineTo(Poly[i].X, Poly[i].Y);
    end;
    lineTo(Poly[0].X, Poly[0].Y);
  end;
end;

procedure TracerTexte(SurBmp: TBitMap; En: TPoint; Text: string; Coul: Tcolor);
begin
  with SurBmp.Canvas do begin
    Brush.Style := bsClear; Font.Name := 'Arial'; Font.Color := Coul; Font.Style := [fsBold];
    TextOut(En.X, En.Y, Text);
  end;
end;

// Quelques "Polygones" :

function PolyAstroide(A, B: single; Centre: TPoint; IncliDeg: Extended): TArrayOfTPoint;
//       A en entre = Largeur et B = Hauteur (N inutilis)
//       dx:=A*cosCube(t); dy:=-B*sinCube(t);
var Theta, dTheta, RR, dxro, dyro, si, co, Alpha: Extended; i: integer;
begin SetLength(Result, 359);
  dTheta := Pi / 180;
  Theta := 0.0;
  for i := 0 to High(Result) do begin
    SinCos(Theta, si, co);
    dxro := A * co * co * co;
    dyro := -B * si * si * si;
    //RR := sqrt(dxro * dxro + dyro * dyro);
    RR := Hypot(dxro, dyro);
    Alpha := arcTan22(dyro, dxro);
    SinCos(Alpha, si, co);
    Result[i].x := round(Centre.X + RR * co);
    Result[i].y := round(Centre.Y + RR * si);
    Theta := Theta + dTheta;
  end;
  if IncliDeg <> 0 then RotaPolygone(Result, Centre, IncliDeg);
end;

function RoEllipse(A, B: single; Ro, nk: Extended; var ThetaRo: Extended): Extended;
//       ici A = Rayon suivant X Largeur (=2*a) et nk = Hauteur (=2*b)
//       x:=a*cos(t); y:=b*sin(t);
var dxro, dyro, si, co: Extended;
begin
  dxro := (Ro / 2) * cos(ThetaRo);
  dyro := (nk / 2) * sin(ThetaRo);
  Result := sqrt(dxro * dxro + dyro * dyro);
  try ThetaRo := arcTan2(dyro, dxro);
  except
  end;
end;

function PolyEllipse(A, B: single; Centre: TPoint; IncliDeg: Extended): TArrayOfTPoint;
var Theta, dTheta, RR, dxro, dyro, si, co, Alpha: Extended; i: integer;
begin SetLength(Result, 359);
  dTheta := Pi / 180;
  Theta := 0.0;
  for i := 0 to High(Result) do begin
    SinCos(Theta, si, co);
    dxro := A * co;
    dyro := B * si;
    RR := Hypot(dxro, dyro);
    Alpha := arcTan22(dyro, dxro);
    SinCos(Alpha, si, co);
    Result[i].x := round(Centre.X + RR * co);
    Result[i].y := round(Centre.Y + RR * si);
    Theta := Theta + dTheta;
  end;
  if IncliDeg <> 0 then RotaPolygone(Result, Centre, IncliDeg);
end;

function FormeF2estdansF1(F2, F1: TArrayOfTPoint): boolean;
var i: integer;
begin
  Result := False;
  for i := 0 to high(F2) do begin
    if not PointDansPolygone(F2[i], F1) then EXIT;
  end;
  Result := True;
end;

var F1, F2: TArrayOfTPoint;

procedure TForm1.bTracerClick(Sender: TObject);
begin
  F1 := PolyEllipse(110, 60, Point(250, 300), 30);
  TracerPoly(Image1.Picture.Bitmap, F1, clRed, 1);
  Image1.Invalidate;

  F2 := PolyAstroide(90, 50, Point(250, 100), 15);
  TracerPoly(Image1.Picture.Bitmap, F2, clBlue, 1);
  Image1.Invalidate;
end;

procedure TForm1.bPremierTestClick(Sender: TObject);
begin
  with Image1.Picture.Bitmap.Canvas do begin
    if not FormeF2estdansF1(F2, F1) then TracerTexte(Image1.Picture.Bitmap, Point(350, 100), 'Astroide hors Ellipse', clRed)
    else TracerTexte(Image1.Picture.Bitmap, Point(350, 300), 'Astroide dans Ellipse', clGreen);
  end;
end;

procedure TForm1.SpeedButton1Click(Sender: TObject);
begin
  TranslaterPolygone(F2, 0, 300 - 100);
  TracerPoly(Image1.Picture.Bitmap, F2, clBlue, 1);
  Image1.Invalidate;
  with Image1.Picture.Bitmap.Canvas do begin
    if not FormeF2estdansF1(F2, F1) then TracerTexte(Image1.Picture.Bitmap, Point(350, 100), 'Astroide hors Ellipse', clRed)
    else TracerTexte(Image1.Picture.Bitmap, Point(350, 300), 'Astroide dans Ellipse', clGreen);
  end;
end;

procedure TForm1.FormResize(Sender: TObject);
var Bmp:TBitMap;
begin
  Bmp:=TBitMap.Create;
  with Bmp do begin
   width:=Image1.Width;  height:=Image1.Height;
  end;
  Image1.Picture.Bitmap.Assign(Bmp);
  Bmp.Free;
end;

end. ////////////////////////////////////////////////////////////////////////////


